/***********************************************************
 * $Id: fancybaselog.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Global logging stuff.
**
*****************************************************************************/

#ifndef FANCYBASE_LOG_H
#define FANCYBASE_LOG_H

#include <fancylog.h>
#include <fancybase.h>

// facility masks:
// facility "fancystdc++"
#define  FANCYBASE_LOG_STDCXX     (1<<0)
// facility "fancylibc"
#define  FANCYBASE_LOG_LIBC       (1<<1)
// facility "fancythread"
#define  FANCYBASE_LOG_THREAD     (1<<2)
// facility "fancycache"
#define  FANCYBASE_LOG_CACHE      (1<<3)
// facility "fancyapp"
#define  FANCYBASE_LOG_APP        (1<<4)
// facility "fancyserializer"
#define  FANCYBASE_LOG_SERIALIZER (1<<5)
// facility "fancyio"
#define  FANCYBASE_LOG_IO         (1<<6)
// facility "fancytimer"
#define  FANCYBASE_LOG_TIMER      (1<<7)
// facility "fancypool"
#define  FANCYBASE_LOG_POOL       (1<<8)

namespace Fancy {
  extern FANCYBASE_API fancylog_service *log_service;

  // parse for logging cmdline args and filter out recognized options.
  void FANCYBASE_API fancylog_parse_arg (int *argc, char **argv);

  inline void vfancylog (int facility,
                         int priority, const char *fmt, va_list ap)
  {
    va_list aq;
    va_copy(aq,ap);
    ::vfancylog(log_service,facility,priority,fmt,aq);
    va_end(aq);
  }

#ifdef __GNUG__
  inline void fancylog (int facility,
                        int priority, const char *fmt, ...)
    __attribute__((format(printf,3,4)));

#endif

  inline void fancylog (int facility,
                        int priority, const char *fmt, ...)
  {
    va_list ap;
    va_start(ap, fmt);
    vfancylog(facility,priority,fmt,ap);
    va_end(ap);
  }
}

#endif // ! FANCYBASE_LOG_H
