/***********************************************************
 * $Id: fancycharsetconverter.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Class for charset conversions.
**
*****************************************************************************/

#ifndef FANCYBASE_CHARSET_CONVERTER_H
#define FANCYBASE_CHARSET_CONVERTER_H

#include <string>
#include <fancyobject.h>

namespace Fancy {

  namespace impl {
    class CharSetConverterImpl;
  }

  /**
   * A CharSetConverter provides easy charset conversions via iconv.
   * It is not threadsafe because the iconv context is stateful, see iconv_open(3).
   */
  class FANCYBASE_API CharSetConverter
  {
  public:
    CharSetConverter(const char *to, const char *from);
    ~CharSetConverter();

    std::string convert(const std::string &in) const;

    CharSetConverter(const CharSetConverter &a);
    CharSetConverter &operator=(const CharSetConverter &a);
  private:
    ObjPtr<impl::CharSetConverterImpl> d;
  };

  class FANCYBASE_API CharToWCharSetConverter
   {
   public:
     CharToWCharSetConverter(const char *cs);
     ~CharToWCharSetConverter();

     std::wstring convert(const std::string &in) const;

     CharToWCharSetConverter(const CharToWCharSetConverter &a);
     CharToWCharSetConverter &operator=(const CharToWCharSetConverter &a);
   private:
     ObjPtr<impl::CharSetConverterImpl> d;
   };

  class FANCYBASE_API WCharToCharSetConverter
   {
   public:
     WCharToCharSetConverter(const char *cs);
     ~WCharToCharSetConverter();

     std::string convert(const std::wstring &in) const;

     WCharToCharSetConverter(const WCharToCharSetConverter &a);
     WCharToCharSetConverter &operator=(const WCharToCharSetConverter &a);
   private:
     ObjPtr<impl::CharSetConverterImpl> d;
   };

}

#endif // ! FANCYBASE_CHARSET_CONVERTER_H
