/***********************************************************
 * $Id: fancyclockval.h 1773 2016-08-08 12:55:30Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas/ITEG
**
** Convenience C++ helper for monotonic clocks.
**
*****************************************************************************/

#ifndef FANCY_CLOCKVAL_H
#define FANCY_CLOCKVAL_H 1

#include <fancybase.h>
#include <fancyint.h>

namespace Fancy {

/**
 * Represent a monotonic real-time clock value.
 *
 * Use like
 *
 * ClockVal start = ClockVal::getClock();
 * ...perform operation...
 * double duration = ClockVal::getDurationSince(start);
 *
 */
class FANCYBASE_API ClockVal {

    int64_t value;
    int64_t info;

public:
    ClockVal(int64_t _val, int64_t _info) : value(_val),info(_info) {}

    /**
     * Return the current real-time clock value.
     */
    static ClockVal getClock();

    /**
     * Return the duration since the start clock value , i.e. the same as
     * ClockVal.getClock() - start
     */
    static double getDurationSince(const ClockVal &start);

    /**
     * @return The value of the clock, which is timespec.tv_sec under Linux
     *         and the actual real-time clock value under Windows.
     */
    int64_t getValue() const { return this->value; }

    /**
     * @return The value of the clock, which is timespec.tv_sec under Linux
     *         and the actual real-time clock resolution under Windows.
     */
    int64_t getInfo() const { return this->info; }
};

/**
 * Return the number of second with fractional parts between @c b and @c a.
 */
double FANCYBASE_API operator-(const ClockVal &a, const ClockVal &b);

/**
 * Add the number of seconds with fractional parts represented by @c b to the clock value @c a.
 */
ClockVal FANCYBASE_API operator+(const ClockVal &a, double b);

/**
 * Add the number of seconds with fractional parts represented by @c a to the clock value @c b.
 */
ClockVal FANCYBASE_API operator+(double a, const ClockVal &b);

}

#endif // ! FANCY_CLOCKVAL_H
