/***********************************************************
 * $Id: fancycollection.h 1108 2014-01-08 10:30:41Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYCOLLECTION_H_
#define FANCYCOLLECTION_H_

#include <fancyint.h>
#include <fancyobject.h>

namespace Fancy {

class Iterator;

class FANCYBASE_API Collection: public virtual Fancy::Object {

public:
    typedef ObjPtr<Collection> ptr_t;

    /**
     * Return the serial ID of the declared component type of this collection.
     * For untyped List instances created through the standardconstructor,
     * NULL is returned.
     *
     *
     */
    virtual const char *getComponentSerialID() const = 0;
    /**
     * Return the serialVersionUID of the declared component type of this collection or
     * @c (int64_t)-1, if the component type is abstract or a non-serializable
     * base class.
     */
    virtual int64_t getComponentSerialVersionUID() const = 0;

    /**
     * @return The number of elements in this collection.
     */
    virtual size_t getSize() const = 0;

    /**
     * @return An iterator, which points to the first element of the collection.
     */
    virtual ObjPtr<Iterator> getBeginIterator() = 0;

    /**
     * @return An iterator, which points past the last element of the collection.
     */
    virtual ObjPtr<Iterator> getEndIterator() = 0;

    /**
     * Remove all elements.
     */
    virtual void clear() = 0;

protected:
    Collection();
    ~Collection();
};

}

#endif /* FANCYCOLLECTION_H_ */
