/***********************************************************
 * $Id: fancycrc.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYCRC_H_
#define FANCYCRC_H_

#include <fancyint.h>
#include <stddef.h>
#include <fancybase.h>

namespace Fancy {

class FANCYBASE_API CRC {
public:
    CRC();
    virtual ~CRC();

    virtual void reset() = 0;
    virtual void update(void const* data, size_t len) = 0;
};

/*! CRC8 with generator polynomial
 *
 *  x^8 + x^5 + x^4 + x + 1
 */
class FANCYBASE_API CRC8 : public CRC {
public:
    CRC8();
    ~CRC8();

    void update(void const* data, size_t len);
    void reset();
    uint8_t get();

private:
    uint8_t crc0;
};

/*!
 * CRC16 with generator polynomial
 *
 * x^16+x^15+x^2+1
 */
class FANCYBASE_API CRC16 : public CRC {
public:
    CRC16();
    ~CRC16();

    void update(void const* data, size_t len);
    void reset();
    uint16_t get();

private:
    uint16_t crc0;
};

/*!
 * CRC32 with generator polynomial
 * x^32 + x^26 + x^23 + x^22 + x^16 + x^12 + x^11 + x^10 + x^8 + x^7 + x^5 + x^4 + x^2 + x + 1
 *
 */
class FANCYBASE_API CRC32 : public CRC {
public:
    CRC32();
    ~CRC32();

    void update(void const* data, size_t len);
    void reset();
    uint32_t get();

private:
    uint32_t crc0;
};

/*!
 * CRC64 with generator polynomial
 *
 * x^64 + x^62 + x^57 + x^55 + x^54 + x^53 + x^52 + x^47 + x^46 + x^45 +
 * x^40 + x^39 + x^38 + x^37 + x^35 + x^33 + x^32 + x^31 + x^29 + x^27 +
 * x^24 + x^23 + x^22 + x^21 + x^19 + x^17 + x^13 + x^12 + x^10 + x^9 +
 * x^7 + x^4 + x + 1
 */
class FANCYBASE_API CRC64 : public CRC {
public:
    CRC64();
    ~CRC64();

    void update(void const* data, size_t len);
    void reset();
    uint64_t get();

private:
    uint64_t crc0;
};

}

#endif /* FANCYCRC_H_ */
