/***********************************************************
 * $Id: fancydatastream.h 1362 2014-11-17 17:37:49Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDATASTREAM_H_
#define FANCYDATASTREAM_H_

#include <fancychannel.h>
#include <fancyint.h>

namespace Fancy {

class Map;

/**
 * An interface for all reading/writing primitives from/to
 * streams.
 */
class FANCYBASE_API DataStream : public virtual Fancy::Object {
public:
    typedef ObjPtr<DataStream> ptr_t;

    virtual bool    readBoolean() = 0;
    virtual int8_t  readByte() = 0;
    virtual int16_t readShort() = 0;
    virtual int32_t readInt() = 0;
    virtual int64_t readLong() = 0;
    virtual wchar_t readChar() = 0;
    virtual float   readFloat() = 0;
    virtual double  readDouble() = 0;
    virtual std::string readUTF() = 0;
    virtual std::string readUTFLong() = 0;

    virtual void writeBoolean(bool b) = 0;
    virtual void writeByte   (int8_t i) = 0;
    virtual void writeShort  (int16_t i) = 0;
    virtual void writeInt    (int32_t i) = 0;
    virtual void writeLong   (int64_t i) = 0;
    virtual void writeChar   (wchar_t c) = 0;
    virtual void writeFloat  (float f) = 0;
    virtual void writeDouble (double d) = 0;
    virtual void writeUTF    (const char *s, int l=-1) = 0;
    virtual void writeUTFLong(const char *s, int l=-1) = 0;

    // FIXME Fancylibs 4: add readU*, writeU*

    virtual void flush() = 0;

    virtual void shutdown(ShutdownOption o) = 0;

    virtual void close() = 0;

protected:

    DataStream() {}
    virtual ~DataStream();
};

}

#endif /* FANCYDATASTREAM_H_ */
