/***********************************************************
 * $Id: fancyiterator.h 1260 2014-06-28 19:27:16Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYITERATOR_H_
#define FANCYITERATOR_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * An iterator on an untyped Object container like @List or @Set
 */
class FANCYBASE_API Iterator: public virtual Fancy::Object {
public:
    typedef ObjPtr<Iterator> ptr_t;

    /**
     * Ac. Fancy::List::Iterator's implementation, this returns wether the iterator points to the first element.
     */
    virtual bool isBegin() const = 0;

    /**
     * Ac. Fancy::List::Iterator's implementation, this returns wether the iterator points to after the last element.
     */
    virtual bool isEnd()   const = 0;

    virtual ObjPtr<Object> getKey() const = 0;

    virtual std::string getKeyString() const;

    virtual ObjPtr<Object> getValue() const = 0;
    virtual void setValue(const ObjPtr<Object> &o) = 0;

    virtual std::string getValueString() const;
    virtual void setValueString(const std::string &o);

    /**
     * Erase the current object and locate the iterator on the next element.
     */
    virtual void erase() = 0;

    /**
     * Has the iterator advance one element.
     * Implementations are NOT obligated to perform validity checks, therefore this might make the iterator unusable without warning.
     */
    virtual void next() = 0;

    /**
     * Has the iterator advance one element backwards.
     * Implementations are NOT obligated to perform validity checks, therefore this might make the iterator unusable without warning.
     */
    virtual void previous() = 0;

protected:
    Iterator();
    virtual ~Iterator();
};

}

#endif /* FANCYITERATOR_H_ */
