/***********************************************************
 * $Id: fancypoolableconnection.h 1424 2015-01-01 12:55:13Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYPOOLABLECONNECTION_H_
#define FANCYPOOLABLECONNECTION_H_

#define FANCY_POOLABLECONNECTION_BEAN "Fancy::PoolableConnection"

#include <fancyobject.h>

namespace Fancy {

/**
 * An implementation of this class represents a poolable connection to a
 * remote resource like a remoting client or a database connection.
 */
class FANCYBASE_API PoolableConnection: public virtual Fancy::Object {
public:

    typedef ObjPtr<PoolableConnection> ptr_t;

    /**
     * @return Whether this instance is currently connected.
     */
    virtual bool isConnected() const = 0;

    /**
     * Disconnect this remote resources. Calling disconnect is
     * needed even, if this instance received a transport error.
     */
    virtual void disconnect() = 0;

    /**
     * connect this remote resource.
     */
    virtual void connect() = 0;

    /**
     * @return Whether this instance has received an error on the transport layer.
     *         If @c true, it is most likely advisable to reconnect this instance.
     */
    virtual bool hasTransportError() const = 0;

protected:
    PoolableConnection() {}
    virtual ~PoolableConnection();
};

class FANCYBASE_API PoolableConnectionFactory: public virtual Fancy::Object {
public:

    typedef ObjPtr<PoolableConnectionFactory> ptr_t;

    /**
     * @return A new poolable connection instance.
     */
    virtual PoolableConnection::ptr_t newConnection() = 0;
protected:
    PoolableConnectionFactory() {}
    virtual ~PoolableConnectionFactory();
};

}

#endif /* FANCYPOOLABLECONNECTION_H_ */
