/***********************************************************
 * $Id: fancyrandom.h 1000 2013-10-07 16:59:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYRANDOM_H_
#define FANCYRANDOM_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * This class
 */
class FANCYBASE_API Random: public Fancy::Object {

public:
    typedef ObjPtr<Random>  ptr_t;

    /**
     * Return a new instance of random. The returned instance is a suitable
     * implementation for you current operating system.
     */
    static ptr_t newInstance();

    /**
     * Generate a sequence of random bytes.
     */
    virtual void generateBytes(uint8_t *buf, int n) = 0;

    /**
     * Generate a 32 bit random number.
     */
    virtual uint32_t generateInt() = 0;

    /**
     * Generate a 64b bit random number.
     */
    virtual uint64_t generateLong() = 0;

protected:
    Random() {}
    virtual ~Random();
};

}

#endif /* FANCYRANDOM_H_ */
