/***********************************************************
 * $Id: fancystringhelper.h 1767 2016-07-12 13:57:39Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCY_STRING_HELPER_H_
#define FANCY_STRING_HELPER_H_

#include <fancybase.h>

#include <string>


namespace Fancy {

namespace StringHelper {


/**
 * Creates a new string from s, removing white space like space, tab, \n, \r
 */
std::string FANCYBASE_API removeWhiteSpace(const char* s);

/**
 * Creates a new string from s, removing white space (like space, tab, \n, \r) at the end
 */
std::string FANCYBASE_API rTrimWhiteSpace(const char* s);

/**
 * Creates a new string from s, putting every char through std::toupper(c,std::locale::classic()), i.e. using legacy C locale.
 * This does not work for umlauts, but is ok for i.e. unifying hex strings.
 */
std::string FANCYBASE_API toUpper(const char* s);

/**
 * Creates a new string from s, putting every char through std::tolower(c,std::locale::classic()), i.e. using legacy C locale.
 * This does not work for umlauts, but is ok for i.e. unifying hex strings.
 */
std::string FANCYBASE_API toLower(const char* s);

/**
 * Masks the core of a password, by overwriting most of it with '*'.
 */
std::string FANCYBASE_API maskPassword(const char* s);


// forget about umlaut upper/lower in C ...


} // namespace BinaryHelper

} // namespace Fancy

#endif /* FANCY_STRING_HELPER_H_ */
