/***********************************************************
 * $Id: fancysystem.h 1483 2015-02-11 20:15:00Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
 **
 ** Author: Christoph Lechleitner / ITEG
 **
 ** Helpers for system tasks
 **
 *****************************************************************************/

#ifndef FANCYBASE_SYSTEM_H
#define FANCYBASE_SYSTEM_H

#include <fancybase.h>
#include <fancyobject.h>
#include <fancyrunnable.h>

//#include <string>


namespace Fancy {

class PollEventHandler;

class FANCYBASE_API SystemController: virtual public Object
{
public:
    typedef ObjPtr<SystemController> ptr_t;

    static ptr_t newInstance();

    virtual int shutdown(bool reboot, int delaySecs=-1) = 0;

};


class FANCYBASE_API ScriptExecutor : public virtual Fancy::Object {

public:

    typedef Fancy::ObjPtr<ScriptExecutor> ptr_t;

    static ptr_t newInstance();

    virtual int execute(const char *binaryPath, const char *argv[]) = 0;

    virtual void setStdOutEater(const Fancy::ObjPtr<Fancy::PollEventHandler> & stdOutEater) = 0;
    virtual void setStdErrEater(const Fancy::ObjPtr<Fancy::PollEventHandler> & stdErrEater) = 0;

    virtual void setForkOverWait(bool _forkOverWait) = 0;

    /**
     * Allow to register a provider for stdin data,
     * e.g. @c SendStringPollHandler
     */
    virtual void setStdinProvider(const Fancy::ObjPtr<Fancy::PollEventHandler> & stdinProvider) = 0;
};


// a runnable that executes the specified binary when run
// particulary useful for performing things "in the background" using Fancy::Timer::runIn_s or the like
class FANCYBASE_API ScriptRunner : public virtual Fancy::Runnable {

public:

    typedef Fancy::ObjPtr<ScriptRunner> ptr_t;

    // don't forget to end argv[] with NULL
    // binaryPath and argv are deep copied
    // see testScriptRunner.cpp for an example
    static ptr_t newInstance(
            const char *binaryPath,
            const char *argv[]);
};


}

#endif // ! FANCYBASE_SYSTEM_H
