/***********************************************************
 * $Id: fancythreadpool.h 1424 2015-01-01 12:55:13Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYTHREADPOOL_H_
#define FANCYTHREADPOOL_H_

#define FANCY_THREADPOOL_BEAN "Fancy::ThreadPool"

#include <fancyobject.h>

namespace Fancy {

class Runnable;

class FANCYBASE_API ThreadPool: public virtual Fancy::Object {
protected:
    ThreadPool();
    virtual ~ThreadPool();
public:
    typedef ObjPtr<ThreadPool> ptr_t;

    /**
     * Return a new default implementation, which holds a minimal number of threads and
     * has an upper limit of threads. Idle threads are suspended after a given timeout.
     *
     * @param minThreads The minimal number of worker threads.
     * @param maxThreads The maximal number of worker threads.
     * @param maxWait The maximal number of waiting runnables, if @c maxThreads workers are busy.
     * @param timeout The timeout in milliseconds after which a thread is suspended.
     * @return An implementation of @c TheadPool.
     */
    static ptr_t newInstance(unsigned minThreads, unsigned maxThreads, unsigned maxWait, unsigned timeout);

    /**
     * Start accepting new runnables. This might incur starting of worker threads
     * depending on your implementation.
     */
    virtual void start() = 0;

    /**
     * Stop accepting new runnables and wait for all other runnables to finish.
     * All currently running threads will be joined after this operation.
     *
     * If not called explicitly, the functionality of this function is called from
     * within the destructor. Long-running destructor code is tedious to debug, so
     * calling this function explicitly is strongly encouraged.
     */
    virtual void stop() = 0;

    /**
     * Schedule the given runnable for execution.
     */
    virtual void execute(const ObjPtr<Runnable> &runnable) = 0;
};

}

#endif /* FANCYTHREADPOOL_H_ */
