/***********************************************************
 * $Id: fancyzipentry.h 1832 2017-06-20 14:55:11Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 ************************************************************/

#ifndef FANCY_ZIPENTRY_H_
#define FANCY_ZIPENTRY_H_

#include <fancyutctimestamp.h>

#include <string>


namespace Fancy {

/**
 * An Object representing an entry in a zip file.
 *
 * Use this object to write data with Fancy::ZipChannel
 */
class FANCYBASE_API ZipEntry : public virtual Object {

private:
	std::string name;
	std::string comment;
	Fancy::UtcTimestamp::ptr_t time;

public:
    typedef ObjPtr<ZipEntry> ptr_t;

    /**
     * Creates a new Instance of ZipEntry, with the name as specifed
     * Note: name must have at most 65.535 Bytes
     */
    static ptr_t newInstance(const std::string &name);

    /**
     * Returns the name of the entry.
     */
    std::string getName() const;

    /**
     * Returns the name of the entry.
     * Note: name must have at most 65.535 Bytes
     */
    void setName(const std::string &name);

    /**
     * Returns the comment string for the entry, or empty string if none.
     */
    std::string getComment() const;

    /**
     * Sets the optional comment string for the entry.
     * Note: comment must have at most 65.535 Bytes
     */
    void setComment(const std::string &comment);

    /**
     * Returns the modification time of the entry, or 0 if not specified.
     */
    Fancy::ObjPtr<Fancy::UtcTimestamp> getTime() const;

    /**
     * Sets the modification time of the entry.
     */
    void setTime(const Fancy::ObjPtr<Fancy::UtcTimestamp> &time);

protected:
    ZipEntry(const std::string &name);
    virtual ~ZipEntry();
};

}

#endif /* FANCY_ZIPENTRY_H_ */
